import { defineComponent, computed, ref, openBlock, createElementBlock, normalizeStyle, unref, normalizeClass, createElementVNode, renderSlot, withModifiers, Fragment, renderList } from "vue";
function isMouseEvent(event) {
  return event instanceof MouseEvent;
}
function isTouchEvent(event) {
  return event instanceof TouchEvent;
}
function rect(width, height) {
  return {
    x: 0,
    y: 0,
    width,
    height
  };
}
function stickStyle(stick, stickSize, scale) {
  const ret = {
    width: `${stickSize / scale.x}px`,
    height: `${stickSize / scale.y}px`
  };
  switch (stick) {
    case "tm":
    case "bm":
      ret.marginLeft = `-${stickSize / scale.x / 2}px`;
      break;
    case "ml":
    case "mr":
      ret.marginTop = `-${stickSize / scale.x / 2}px`;
      break;
  }
  return ret;
}
function getPageEvent(ev) {
  if (isMouseEvent(ev)) {
    return ev;
  }
  return ev.touches[0];
}
function limitRotateAngle(angle) {
  if (angle < -90) {
    return -90;
  }
  if (angle > 90) {
    return 90;
  }
  return angle;
}
function hasOwnProperty(obj, prop) {
  return Object.prototype.hasOwnProperty.call(obj, prop);
}
function setStyle(el, styles) {
  for (const key in styles) {
    if (hasOwnProperty(styles, key) && styles[key] !== void 0) {
      el.style[key] = styles[key];
    }
  }
}
function createResizeRect(size) {
  const stickSize = size * 3;
  return (stick, delta, base) => {
    const ret = rect(base.width, base.height);
    ret.x = base.x;
    ret.y = base.y;
    switch (stick) {
      case "bl":
        ret.width = Math.max(-delta.x + base.width, stickSize);
        ret.height = delta.y + base.height;
        ret.x = base.width - ret.width + base.x;
        break;
      case "bm":
        ret.height = Math.max(delta.y + base.height, stickSize);
        break;
      case "br":
        ret.width = Math.max(delta.x + base.width, stickSize);
        ret.height = Math.max(delta.y + base.height, stickSize);
        break;
      case "tl":
        ret.width = Math.max(-delta.x + base.width, stickSize);
        ret.height = Math.max(-delta.y + base.height, stickSize);
        ret.x = base.width - ret.width + base.x;
        ret.y = base.height - ret.height + base.y;
        break;
      case "tm":
        ret.height = Math.max(-delta.y + base.height, stickSize);
        ret.y = base.height - ret.height + base.y;
        break;
      case "tr":
        ret.width = Math.max(delta.x + base.width, stickSize);
        ret.height = Math.max(-delta.y + base.height, stickSize);
        ret.y = base.height - ret.height + base.y;
        break;
      case "ml":
        ret.width = Math.max(-delta.x + base.width, stickSize);
        ret.x = base.width - ret.width + base.x;
        break;
      case "mr":
        ret.width = Math.max(delta.x + base.width, stickSize);
        break;
    }
    return ret;
  };
}
var vueDragResize_vue_vue_type_style_index_0_lang = "";
const _hoisted_1 = ["onMousedown", "onTouchstart"];
const _sfc_main = /* @__PURE__ */ defineComponent({
  props: {
    x: { default: 0 },
    y: { default: 0 },
    z: { default: 0 },
    width: { default: 100 },
    height: { default: 100 },
    rotate: { default: 0 },
    immediate: { type: Boolean, default: false },
    parentScaleX: { default: 1 },
    parentScaleY: { default: 1 },
    stickSize: { default: 10 },
    sticks: { default: () => ["tl", "tm", "tr", "mr", "br", "bm", "bl", "ml"] },
    resizable: { type: Boolean, default: true },
    rotatable: { type: Boolean, default: true },
    active: { type: Boolean, default: false }
  },
  emits: ["dragging", "dragEnd", "resizing", "resizeEnd", "rotating", "rotateEnd"],
  setup(__props, { emit: emits }) {
    const props = __props;
    const resizeRect = createResizeRect(props.stickSize);
    const vdrStyle = computed(() => ({
      left: `${props.x}px`,
      top: `${props.y}px`,
      zIndex: props.z
    }));
    const bodyStyle = computed(() => ({
      width: `${props.width}px`,
      height: `${props.height}px`,
      transform: `rotate(${props.rotate}deg)`
    }));
    const rotatePointerStyle = computed(() => ({
      transform: `rotate(${props.rotate}deg)`
    }));
    const vdrRef = ref();
    const bodyRef = ref();
    const rotatePointerRef = ref();
    const lastPoint = {
      x: 0,
      y: 0
    };
    let rectBox = rect(props.width, props.height);
    let resizeStick = "";
    let rotateAngle = 0;
    function dragging(ev) {
      if (!vdrRef.value) {
        return;
      }
      const { pageX, pageY } = getPageEvent(ev);
      const deltaX = pageX - lastPoint.x;
      const deltaY = pageY - lastPoint.y;
      if (props.immediate) {
        lastPoint.x = pageX;
        lastPoint.y = pageY;
        rectBox.x = deltaX + rectBox.x;
        rectBox.y = deltaY + rectBox.y;
        emits("dragging", rectBox);
        return;
      }
      rectBox.x = deltaX + props.x;
      rectBox.y = deltaY + props.y;
      setStyle(vdrRef.value, {
        left: `${rectBox.x}px`,
        top: `${rectBox.y}px`
      });
    }
    function dragend() {
      emits("dragEnd", rectBox);
      lastPoint.x = 0;
      lastPoint.y = 0;
      document.removeEventListener("mousemove", dragging);
      document.removeEventListener("mouseup", dragend);
      document.removeEventListener("touchmove", dragging);
      document.removeEventListener("touchend", dragend);
    }
    function dragstart(ev) {
      if (!props.active) {
        return;
      }
      rectBox.x = props.x;
      rectBox.y = props.y;
      const { pageX, pageY } = getPageEvent(ev);
      lastPoint.x = pageX;
      lastPoint.y = pageY;
      if (isMouseEvent(ev)) {
        document.addEventListener("mousemove", dragging);
        document.addEventListener("mouseup", dragend);
        return;
      }
      document.addEventListener("touchmove", dragging);
      document.addEventListener("touchend", dragend);
    }
    function resizing(ev) {
      if (!vdrRef.value || !bodyRef.value) {
        return;
      }
      const { pageX, pageY } = getPageEvent(ev);
      const deltaX = pageX - lastPoint.x;
      const deltaY = pageY - lastPoint.y;
      if (props.immediate) {
        lastPoint.x = pageX;
        lastPoint.y = pageY;
        rectBox = resizeRect(resizeStick, {
          x: deltaX,
          y: deltaY
        }, rectBox);
        emits("resizing", rectBox);
        return;
      }
      rectBox = resizeRect(resizeStick, {
        x: deltaX,
        y: deltaY
      }, props);
      setStyle(vdrRef.value, {
        left: `${rectBox.x}px`,
        top: `${rectBox.y}px`
      });
      setStyle(bodyRef.value, {
        width: `${rectBox.width}px`,
        height: `${rectBox.height}px`
      });
    }
    function resizeEnd() {
      emits("resizeEnd", rectBox);
      lastPoint.x = 0;
      lastPoint.y = 0;
      document.removeEventListener("mousemove", resizing);
      document.removeEventListener("mouseup", resizeEnd);
      document.removeEventListener("touchmove", resizing);
      document.removeEventListener("touchend", resizeEnd);
    }
    function resizeStart(ev, stick) {
      resizeStick = stick;
      const { pageX, pageY } = getPageEvent(ev);
      lastPoint.x = pageX;
      lastPoint.y = pageY;
      if (isMouseEvent(ev)) {
        document.addEventListener("mousemove", resizing);
        document.addEventListener("mouseup", resizeEnd);
        return;
      }
      document.addEventListener("touchmove", resizing);
      document.addEventListener("touchend", resizeEnd);
    }
    function rotating(ev) {
      if (!bodyRef.value || !rotatePointerRef.value) {
        return;
      }
      const { pageX, pageY } = getPageEvent(ev);
      const deltaX = Math.min(Math.max(pageX - lastPoint.x, -50), 50);
      const deltaAngle = Math.asin(deltaX / 50) * 180 / Math.PI;
      if (props.immediate) {
        lastPoint.x = pageX;
        lastPoint.y = pageY;
        rotateAngle = limitRotateAngle(rotateAngle + deltaAngle);
        emits("rotating", rotateAngle);
        return;
      }
      rotateAngle = limitRotateAngle(props.rotate + deltaAngle);
      setStyle(bodyRef.value, {
        transform: `rotate(${rotateAngle}deg)`
      });
      setStyle(rotatePointerRef.value, {
        transform: `rotate(${rotateAngle}deg)`
      });
    }
    function rotateEnd() {
      emits("rotateEnd", rotateAngle);
      lastPoint.x = 0;
      lastPoint.y = 0;
      document.removeEventListener("mousemove", rotating);
      document.removeEventListener("mouseup", rotateEnd);
      document.removeEventListener("touchmove", rotating);
      document.removeEventListener("touchend", rotateEnd);
    }
    function rotateStart(ev) {
      rotateAngle = props.rotate;
      const { pageX, pageY } = getPageEvent(ev);
      lastPoint.x = pageX;
      lastPoint.y = pageY;
      if (isMouseEvent(ev)) {
        document.addEventListener("mousemove", rotating);
        document.addEventListener("mouseup", rotateEnd);
        return;
      }
      document.addEventListener("touchmove", rotating);
      document.addEventListener("touchend", rotateEnd);
    }
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        ref_key: "vdrRef",
        ref: vdrRef,
        style: normalizeStyle(unref(vdrStyle)),
        class: normalizeClass(["vdr"]),
        onMousedown: dragstart,
        onTouchstart: dragstart
      }, [
        createElementVNode("div", {
          ref_key: "bodyRef",
          ref: bodyRef,
          style: normalizeStyle(unref(bodyStyle))
        }, [
          renderSlot(_ctx.$slots, "default")
        ], 4),
        createElementVNode("div", {
          class: normalizeClass(["vdr-drag", { active: props.active }])
        }, null, 2),
        createElementVNode("div", {
          class: normalizeClass(["vdr-rotate", props.active && props.rotatable ? "" : "not-ratable"])
        }, [
          createElementVNode("div", {
            ref_key: "rotatePointerRef",
            ref: rotatePointerRef,
            class: "vdr-rotate-pointer",
            style: normalizeStyle(unref(rotatePointerStyle)),
            onMousedown: _cache[0] || (_cache[0] = withModifiers(($event) => rotateStart($event), ["stop", "prevent"])),
            onTouchstart: _cache[1] || (_cache[1] = withModifiers(($event) => rotateStart($event), ["stop", "prevent"]))
          }, null, 36)
        ], 2),
        (openBlock(true), createElementBlock(Fragment, null, renderList(props.sticks, (stick) => {
          return openBlock(), createElementBlock("div", {
            key: stick,
            class: normalizeClass(["vdr-stick", ["vdr-stick-" + stick, props.resizable && props.active ? "" : "not-resizable"]]),
            style: normalizeStyle(unref(stickStyle)(stick, props.stickSize, {
              x: props.parentScaleX,
              y: props.parentScaleY
            })),
            onMousedown: withModifiers(($event) => resizeStart($event, stick), ["stop", "prevent"]),
            onTouchstart: withModifiers(($event) => resizeStart($event, stick), ["stop", "prevent"])
          }, null, 46, _hoisted_1);
        }), 128))
      ], 36);
    };
  }
});
export { createResizeRect, _sfc_main as default, getPageEvent, hasOwnProperty, isMouseEvent, isTouchEvent, limitRotateAngle, rect, setStyle, stickStyle };
