export function isMouseEvent(event) {
    return event instanceof MouseEvent;
}
export function isTouchEvent(event) {
    return event instanceof TouchEvent;
}
export function rect(width, height) {
    return {
        x: 0,
        y: 0,
        width,
        height,
    };
}
export function stickStyle(stick, stickSize, scale) {
    const ret = {
        width: `${stickSize / scale.x}px`,
        height: `${stickSize / scale.y}px`,
    };
    switch (stick) {
        case 'tm':
        case 'bm':
            ret.marginLeft = `-${stickSize / scale.x / 2}px`;
            break;
        case 'ml':
        case 'mr':
            ret.marginTop = `-${stickSize / scale.x / 2}px`;
            break;
        default:
            break;
    }
    return ret;
}
export function getPageEvent(ev) {
    if (isMouseEvent(ev)) {
        return ev;
    }
    return ev.touches[0];
}
export function limitRotateAngle(angle) {
    if (angle < -90) {
        return -90;
    }
    if (angle > 90) {
        return 90;
    }
    return angle;
}
export function hasOwnProperty(obj, prop) {
    return Object.prototype.hasOwnProperty.call(obj, prop);
}
export function setStyle(el, styles) {
    for (const key in styles) {
        if (hasOwnProperty(styles, key) && styles[key] !== undefined) {
            el.style[key] = styles[key];
        }
    }
}
export function createResizeRect(size) {
    const stickSize = size * 3;
    return (stick, delta, base) => {
        const ret = rect(base.width, base.height);
        ret.x = base.x;
        ret.y = base.y;
        switch (stick) {
            case 'bl':
                ret.width = Math.max(-delta.x + base.width, stickSize);
                ret.height = delta.y + base.height;
                ret.x = base.width - ret.width + base.x;
                break;
            case 'bm':
                ret.height = Math.max(delta.y + base.height, stickSize);
                break;
            case 'br':
                ret.width = Math.max(delta.x + base.width, stickSize);
                ret.height = Math.max(delta.y + base.height, stickSize);
                break;
            case 'tl':
                ret.width = Math.max(-delta.x + base.width, stickSize);
                ret.height = Math.max(-delta.y + base.height, stickSize);
                ret.x = base.width - ret.width + base.x;
                ret.y = base.height - ret.height + base.y;
                break;
            case 'tm':
                ret.height = Math.max(-delta.y + base.height, stickSize);
                ret.y = base.height - ret.height + base.y;
                break;
            case 'tr':
                ret.width = Math.max(delta.x + base.width, stickSize);
                ret.height = Math.max(-delta.y + base.height, stickSize);
                ret.y = base.height - ret.height + base.y;
                break;
            case 'ml':
                ret.width = Math.max(-delta.x + base.width, stickSize);
                ret.x = base.width - ret.width + base.x;
                break;
            case 'mr':
                ret.width = Math.max(delta.x + base.width, stickSize);
                break;
            default:
                break;
        }
        return ret;
    };
}
//# sourceMappingURL=utils.js.map